#!/bin/sh

ACTION=${1}
shift 1

get_image() {
  IMAGE_NAME="lscr.io/linuxserver/mastodon:latest"
}

do_install_detail() {
  local config=`uci get mastodon.@mastodon[0].config_path 2>/dev/null`
  local port=`uci get mastodon.@mastodon[0].port 2>/dev/null`
  local IMAGE_NAME=`uci get mastodon.@mastodon[0].image_name 2>/dev/null`
  local username=`uci get mastodon.@mastodon[0].username 2>/dev/null`
  local password=`uci get mastodon.@mastodon[0].password 2>/dev/null`

  #Generate the generic environment variables for the docker-compose

  GEN_PASS=$(< /dev/urandom tr -dc A-Za-z0-9 2>/dev/null | head -c14; echo)
  GEN_PASS2=$(< /dev/urandom tr -dc A-Za-z0-9 2>/dev/null | head -c14; echo)


# rake secrets
secret=$(docker run --rm -it --entrypoint /bin/bash lscr.io/linuxserver/mastodon generate-secret)
base=$(docker run --rm -it --entrypoint /bin/bash lscr.io/linuxserver/mastodon generate-secret)

output=$(docker run --rm -it --entrypoint /bin/bash lscr.io/linuxserver/mastodon generate-vapid)

private_key=$(echo "$output" | grep VAPID_PRIVATE_KEY | awk -F= '{print $2}')
public_key=$(echo "$output" | grep VAPID_PUBLIC_KEY | awk -F= '{print $2}')

   
  if [ -z "$config" ]; then
      echo "config path is empty!"
      exit 1
  fi

  [ -z "$port" ] && port=""
  [ -z "$IMAGE_NAME" ] && IMAGE_NAME=lscr.io/linuxserver/mastodon:latest

rm -R /opt/docker2/compose/mastodon
mkdir /opt/docker2/compose/mastodon

touch /opt/docker2/compose/mastodon/docker-compose.yml
cat > /opt/docker2/compose/mastodon/docker-compose.yml 

  # Create Docker Compose file with custom variables

cat > /opt/docker2/compose/mastodon/docker-compose.yml <<EOF
version: "2.1"
services:
  mastodon:
    image: $IMAGE_NAME
    container_name: mastodon
    environment:
      - PUID=1000
      - PGID=1000
      - TZ=Etc/UTC
      - LOCAL_DOMAIN=$config
      - REDIS_HOST=redis
      - REDIS_PORT=6379
      - DB_HOST=db
      - DB_USER=toot
      - DB_NAME=mastodondb
      - DB_PASS=$GEN_PASS
      - DB_PORT=5432
      - ES_ENABLED=false
      - SECRET_KEY_BASE=$secret
      - OTP_SECRET=$base
      - VAPID_PRIVATE_KEY=$private_key
      - VAPID_PUBLIC_KEY=$public_key
      - SMTP_SERVER=mail.example.com
      - SMTP_PORT=25
      - SMTP_LOGIN=
      - SMTP_PASSWORD=
      - SMTP_FROM_ADDRESS=notifications@example.com
      - S3_ENABLED=false
      #- WEB_DOMAIN=mastodon.example.com #optional
      #- ES_HOST=es #optional
      #- ES_PORT=9200 #optional
      #- ES_USER=elastic #optional
      #- ES_PASS=elastic #optional
      #- S3_BUCKET= #optional
      #- AWS_ACCESS_KEY_ID= #optional
      #- AWS_SECRET_ACCESS_KEY= #optional
      #- S3_ALIAS_HOST= #optional
      #- SIDEKIQ_ONLY=false #optional
      #- SIDEKIQ_QUEUE= #optional
      #- SIDEKIQ_DEFAULT=false #optional
      #- SIDEKIQ_THREADS=5 #optional
      #- DB_POOL=5 #optional
    volumes:
      - /opt/docker2/compose/mastodon/config:/config
    ports:
      - "$port:80"
      - "4643:443"
    depends_on:
      - db
      - redis
    restart: unless-stopped

  db:
    image: postgres:14
    container_name: db
    volumes:
      - /opt/docker2/compose/mastodon/data/postgres:/var/lib/postgresql/data
    environment:
      - POSTGRES_DB=mastodondb
      - POSTGRES_USER=toot
      - POSTGRES_PASSWORD=$GEN_PASS
    restart: unless-stopped

  redis:
    image: redis
    container_name: redis
    expose:
      - "6379"
    restart: unless-stopped
EOF

  docker-compose -f /opt/docker2/compose/mastodon/docker-compose.yml up -d

 
 
  # Add a new list option to the "shortcutmenu" configuration file
  uci add shortcutmenu lists
  uci set shortcutmenu.@lists[-1].webname="$IMAGE_NAME"
  uci set shortcutmenu.@lists[-1].weburl="$LAN_IP:$port"
  uci set shortcutmenu.@lists[-1].webpath="/"
  uci commit shortcutmenu

}


usage() {
  echo "usage: $0 sub-command"
  echo "where sub-command is one of:"
  echo "      install                Install mastodon"
  echo "      upgrade                Upgrade mastodon"
  echo "      rm/start/stop/restart  Remove/Start/Stop/Restart mastodon"
  echo "      status                 mastodon status"
  echo "      port                   mastodon port"
}

case ${ACTION} in
  "install"|"upgrade")
    get_image
    do_install_detail
  ;;
  "rm")
    get_image
    CONTAINER_IDS=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.ID}}')
    echo "Stopping and removing containers..."
    for ID in $CONTAINER_IDS; do
      docker stop $ID
      docker rm $ID
    done
    docker rmi -f $IMAGE_NAME
  ;;
  "start"|"stop"|"restart")
    get_image
    CONTAINER_IDS=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.ID}}')
    for ID in $CONTAINER_IDS; do
      docker ${ACTION} ${ID}
    done
  ;;
  "status")
    get_image
    CONTAINER_NAMES=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.Names}}')
    docker ps --all -f "name=${CONTAINER_NAMES}" --format '{{.Status}}'
  ;;
  "port")
    get_image
    CONTAINER_NAMES=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.Names}}')
    docker ps --all -f "name=${CONTAINER_NAMES}" --format '{{.Ports}}' | grep -om1 '0.0.0.0:[0-9]*' | sed 's/0.0.0.0://'
  ;;
  *)
    usage
    exit 1
  ;;
esac
